'use strict'

class FileSaver {
  
  static get instance() {
    if ( ! this._instance ) { 
      this._singletonEnforcer = Symbol();
      this._instance = new FileSaver(this._singletonEnforcer);
    }
    return this._instance;
  }
  
  constructor(enforcer) {
    if (!enforcer || enforcer != FileSaver._singletonEnforcer)
      throw new Error("cannot construct singleton. use static method 'instance' instead");
    
    this._textFile = null;
  }
  
  
  makeTextFile(text) {
    let data = new Blob([text], {type:'text/plain'});

    // If we are replacing a previously generated file we need to
    // manually revoke the object URL to avoid memory leaks.    if (this._textFile !== null) {
      window.URL.revokeObjectURL(this._textFile);
    
    this._textFile = window.URL.createObjectURL(data);
    
    return this._textFile;
  }
  
  saveTextFile(text, filename){
    let link = document.createElement("a");
    link.setAttribute("download", filename);
    link.href = this.makeTextFile(text);
    document.body.appendChild(link);
    
    window.requestAnimationFrame(function() {
      var event = new MouseEvent("click");
      link.dispatchEvent(event);
      document.body.removeChild(link);
    });
  }
}


$(()=>{
  let codeMirrorEditor = null;
  let codeMirrorOutput = null;
  let parser = null;
  
  Parser.load("parser.peg", (res)=>{
    parser = res;
  });
  
  function initCodeMirror() {
    let $editor = $('#editor');
    
    codeMirrorEditor = CodeMirror($editor[0], {
      // value: "hello world",
      lineNumbers: true,
      theme: "zenburn",
      lineWrapping: true,
    });
    
    codeMirrorOutput = CodeMirror($('#output')[0], {
      // theme: "zenburn",
      lineWrapping: true,
      theme: "zenburn",
      readOnly: true,
    });
  }
  
  
  
  let horizontalLayout = true;
  
  function changeLayout(horizontal) {
    if (horizontal) {
      $('#container').removeClass('vertical');
      
    }
    else {
      $('#container').addClass('vertical');
    }
  }
  
  function logSize($elem, name) {
    // if (name)
    //   console.group(name);
    // console.log("width x height", $elem.width() + " x " + $elem.height());
    // console.log("inner         ", $elem.innerWidth() + " x " + $elem.innerHeight());
    // console.log("outer         ", $elem.outerWidth() + " x " + $elem.outerHeight());
    // console.log("outer (margin)", $elem.outerWidth(true) + " x " + $elem.outerHeight(true));
    if (name)
      console.groupEnd();
  }
  
  function computeContainerHeight() {
    // logSize($(window), "window");
    // logSize($('#toolbar'), "toolbar");
    // logSize($('#container'), "container");
    
    $('#container').outerHeight($(window).height() - $('#header').outerHeight(true) - 16);
    
    let $container = $('#container');
    let $editor = $('#editor');
    let $output = $('#output');
    
    logSize($('#container'), "container");
    logSize($editor, "editor");
    logSize($output, "editor");
    
    if ($('#container').hasClass('vertical')) {
      // console.log('resize vertical ?');
      
    }
    else {
      // console.log('resize horizontal?');
      let leftMargin = $editor.css("margin-left");
      $editor.css("margin-right", leftMargin);
      $output.css("margin-left", "0px");
      $output.css("margin-right", leftMargin);
      
      leftMargin = parseInt(leftMargin);
      
      let width = Math.ceil(($container.innerWidth() - 3 * (leftMargin) )  / 2) - 3;
      // console.log(leftMargin, $container.innerWidth(), width);


      $editor.outerWidth(width);
      $output.outerWidth(width);
      
      // console.log($container.innerHeight());
      let height = $container.innerHeight() - parseInt($editor.css("margin-top"))- parseInt($editor.css("margin-bottom"));
      $editor.outerHeight(height);
      $output.outerHeight(height);
    }
  }
  
  function computeSize() {
    let $window = $(windows);
    let $container = $('#container');
    let $editor = $('#editor');
    let $output = $('#output');
    
    
  }
  
  $("#btn-clear").click(()=>{
    codeMirrorEditor.setValue("");
    codeMirrorOutput.setValue("");
  });
  
  
  $("#btn-run").click(()=>{
    console.assert(parser != null);
    
    try {
      let {output} = parser.parse(codeMirrorEditor.getValue());
      codeMirrorOutput.setValue(output);
    }
    catch (err) {
      codeMirrorOutput.setValue(err.toString());
      // codeMirrorEditor.setSelection( {
          // line: err.location.start.line-1, 
          // ch: err.location.start.column-1
        // },{
          // line: err.location.end.line-1, 
          // ch: err.location.end.column-1
        // }
      // );
      codeMirrorEditor.setCursor(err.location.start.line-1, err.location.start.column-1);
      codeMirrorEditor.focus();
    }
  });
  
  $("#toolbar button").button();

  $("#toolbar input[type=radio]").checkboxradio({
    icon: false,
  });
  
  $("#toolbar input[name='radio-layout']").on("change", function(){
    let horizontal = this.id == "radio-horizontal";
    
    changeLayout(horizontal);
  });
  
  $("#btn-load").click(()=>{
    $("#load-file").val("");
    $("#load-file").click();
  });
  
  $("#load-file").on("change", (evt)=>{
    console.log("change event");
    let files = evt.target.files;
    if (! files || files.length == 0)
      return;
    
    let file = files[0];
    // console.log("type", file);
    // console.log("sqf ?", file.name.endsWith(".sqf"));
    
    let reader = new FileReader();
    
    reader.onload = (e) => {
      console.log("file loaded");
      codeMirrorEditor.setValue(e.target.result);
      $('#btn-run').click();
    }
    
    reader.readAsText(file);
    
    
    
  });
  
  $("#btn-save").click(()=>{
    FileSaver.instance.saveTextFile(codeMirrorEditor.getValue(), "test.sqf");
  });
  

  changeLayout(true);
  
  $(window).resize(function() {
    computeContainerHeight();
  });

  $(window).trigger('resize');  
  
  initCodeMirror();
});