class AlgebraError {
  static _initConstants(){
    AlgebraError.FORBIDDEN_OPERATOR = Symbol("forbidden comparison operator");
    AlgebraError.FORBIDDEN_OPERATION = Symbol("forbidden algebra operation");
    AlgebraError.INCOMPATIBLE_SCHEMA = Symbol("incompatible schema");
    AlgebraError.INTERNAL = Symbol("internal error");
    AlgebraError.INVALID_ATTRIBUTE = Symbol("invalid attribute name");
    AlgebraError.INVALID_ATTRIBUTE_VALUE = Symbol("invalid attribute value");
    AlgebraError.INVALID_CONDITION = Symbol("invalid condition");
    AlgebraError.INVALID_EXPRESSION = Symbol("invalid expression");
    AlgebraError.INVALID_INDEX = Symbol("invalid index");
    AlgebraError.INVALID_RENAME_MAP = Symbol("invalid rename mapping");
    AlgebraError.INVALID_SCHEMA = Symbol("invalid schema");
    AlgebraError.INVALID_TUPLE = Symbol("invalid tuple");
    AlgebraError.INVALID_TUPLESET = Symbol("invalid tuple set");
  }
  
  constructor ( type = null, details = null ){
    AlgebraError._checkType(type);
    this._type = type;
    this._details = details;
    
  }

  static _checkType(type) {
    switch ( type ) {
      case AlgebraError.FORBIDDEN_OPERATOR:
      case AlgebraError.FORBIDDEN_OPERATION:
      case AlgebraError.INCOMPATIBLE_SCHEMA:
      case AlgebraError.INTERNAL:
      case AlgebraError.INVALID_ATTRIBUTE:
      case AlgebraError.INVALID_ATTRIBUTE_VALUE:
      case AlgebraError.INVALID_CONDITION :
      case AlgebraError.INVALID_EXPRESSION:
      case AlgebraError.INVALID_INDEX:
      case AlgebraError.INVALID_RENAME_MAP:
      case AlgebraError.INVALID_SCHEMA:
      case AlgebraError.INVALID_TUPLE:
      case AlgebraError.INVALID_TUPLESET:
        break;
      default:
        throw new Error(`error type ${type} should be a AlgebraError.XXX constant`);
    }
  }
  
  get type() {
    return this._type;
  }
  
  get details() {
    return this._details;
  }

  get description() {
    let res = "";

    switch(this._type) {
      case AlgebraError.FORBIDDEN_OPERATOR:
          res += "forbidden comparison operator in select condition";
          break;
          
      case AlgebraError.FORBIDDEN_OPERATION:
          res += "forbidden algebra operation according to @forbid directive";
          break;
          
      case AlgebraError.INCOMPATIBLE_SCHEMA:
          res += "incompatible schemas";
          break;
          
      case AlgebraError.INTERNAL:
          res += "internal error";
          break;
          
      case AlgebraError.INVALID_ATTRIBUTE:
          res += "invalid attribute";
          break;
          
      case AlgebraError.INVALID_ATTRIBUTE_VALUE:
          res += "invalid attribute value";
          break;
          
      case AlgebraError.INVALID_CONDITION :
          res += "invalid condition";
          break;
          
      case AlgebraError.INVALID_EXPRESSION:
          res += "invalid expression";
          break;
          
      case AlgebraError.INVALID_INDEX:
          res += "invalid index";
          break;
          
      case AlgebraError.INVALID_RENAME_MAP:
          res += "invalid rename mapping";
          break;
          
      case AlgebraError.INVALID_SCHEMA:
          res += "invalid schema";
          break;
          
      case AlgebraError.INVALID_TUPLE:
          res += "invalid tuple";
          break;
          
      case AlgebraError.INVALID_TUPLESET:
          res += "invalid tuple set";
          break;
          
      default:
        throw Error("Unexpected Algebra error type : " + this._type.toString());
        break;
    }
    
    if (this.details != null) 
      res += `: ${this._details} `;
    
    return res;
  }
  
  toString(){
    return `[AlgebraError] ${this.description}`;
  }
}


AlgebraError._initConstants();