'use strict'

class Difference extends Expression {
  constructor( {
      leftExpression = null, 
      rightExpression = null} = {} ) {
        
    if (ForbiddenOperations.instance.isForbidden("D"))
      throw new AlgebraError(AlgebraError.FORBIDDEN_OPERATION, "Difference");
        
    super();
    
    this._left = leftExpression;
    this._right = rightExpression;

    this._check();
    this._setSchema(this._left.schema);

    super._expressionString = `${leftExpression.toExpressionString(this).trim()} - ${rightExpression.toExpressionString(this).trim()}`;
  }
  
  get level() {
    return 2;
  }
  
  _evaluate() {
    let tuplesLeft = Immutable.Seq(this._left.tupleSet).sort((o1,o2) => o1.compareTo(o2));
    
    let indexes = this._left.schema.getAttributeIndexes(this._right.schema);
    
    let tuplesRight = Immutable.Seq(this._right.tupleSet)
        .map(tuple => tuple.cloneReordered(indexes))
        .sort((o1,o2) => o1.compareTo(o2));

    let tuples = [];
    let iL = 0;
    let iR = 0;
    
    while ( iL < tuplesLeft.size && iR < tuplesRight.size ) {
      let tL = tuplesLeft.get(iL);
      let tR = tuplesRight.get(iR);
      let compare = tL.compareTo(tR);
      
      if (compare > 0) {
        iR++;
      }
      else if (compare == 0) {
        iL++;
        iR++;
      }
      else {
        tuples.push(tL);
        iL++;
      }
    }
    
    while (iL < tuplesLeft.size) {
      let tL = tuplesLeft.get(iL);
        tuples.push(tL);
        iL++;
    }
    
    delete this._left;
    delete this._right;
    
    this._setTupleSet(new TupleSet(tuples));
  }
  
  _check() {
    if ( !(this._left instanceof Expression) ) 
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, "left operand");
    
    if ( !(this._right instanceof Expression) ) 
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, "right operand");
    
    if (! this._left.schema.equals(this._right.schema))
      throw new AlgebraError(
        AlgebraError.INCOMPATIBLE_SCHEMA, 
        `left ${this._left.schema.toString()} - right ${this._right.schema.toString()}`);
  }
}