'use strict'

class Expression {
  constructor(){
    this._name = null;
    this._schema = null;
    this._tupleSet = null;
    this._expressionString = null;
  }
  
  clone() {
    let res = new Expression();
    res._schema = this._schema;
    res._tupleSet = this._tupleSet;
  }
  
  
  get tupleSet(){
    if (this._tupleSet === null) {
      if (this._evaluate == undefined)
        throw new AlgebraError(AlgebraError.INTERNAL, "derived class must implement abstract '_evaluate' method");
      this._evaluate();

      if (this._tupleSet === null)
        throw new AlgebraError(AlgebraError.INTERNAL, "_evaluate() should set this._tupleSet");
    }
    
    return this._tupleSet;
  }
  
  get schema(){
    return this._schema;
  }
  
  get name(){
    return this._name;
  }
  
  set name(value){
    if (this._name !== null)
      throw new AlgebraError(
        AlgebraError.INTERNAL,
        `Cannot rename expression '${this._name}' to '${value}]'` );
    
    this._name = value;
  }
  
  toSQF() {
    throw new AlgebraError(AlgebraError.INTERNAL, "todo");
  }
  
  toString() {
    let name = this.name; 
    
    if (name == null) {
      Expression._counter++;
      name = "unnamed_expression_" + Expression._counter;
    }
      
    let res = "@let " + name + ' = @relation[' + this.schema.toString() + "] {";
    
    if (this.tupleSet.size == 0) {
      res += "}";
    }
    else {
      res += "\n";
      for (let t of this.tupleSet)
        res += "\t" + t.toString() + "\n";
      res += "}";
    }
    
    return res;
  }

  toTypeString() {
    throw new AlgebraError(AlgebraError.INTERNAL, "todo");
  }
  
  toExpressionString(parent) {
    if (! (parent instanceof Expression))
      return this._expressionString;
    let res = this.name || this._expressionString;
    let atomic = this.name || this.atomic;
    
    if (parent.level < this.level || ! atomic )
      return "(" + res + ")";
    else 
      return " " + res;
  }
  
  _linkTuplesToSchema(){
    for (let tuple of this._tupleSet) {
      tuple.schema = this._schema;
    }
  }
  
  _setTupleSet(tupleSet){
    if (this._tupleSet !== null)
      throw new AlgebraError(AlgebraError.INTERNAL, "tupleSet can only be set once");

    if (tupleSet === null || ! (tupleSet instanceof TupleSet))
      throw new AlgebraError(AlgebraError.INTERNAL, "invalid TupleSet");
    
    this._tupleSet = tupleSet;
    this._linkTuplesToSchema();
  }
  
  _setSchema(schema){
    if (this._schema !== null)
      throw new AlgebraError(AlgebraError.INTERNAL, "schema can only be set once");
   
    if ( schema === null || ! (schema instanceof Schema) )
      throw new AlgebraError(AlgebraError.INTERNAL, "invalid Schema");
    
    this._schema = schema;
  }
  
  static resetCounter() {
    Expression._counter = 0;
  }
}


Expression.resetCounter();
