'use strict'

// singleton
// defines the allowed comparison operators used in select condition
class ExtendedOperators {
  
  static initConstants() {
    this._singletonEnforcer = Symbol();
  }
  
  static get instance() {
    if ( ! this._instance ) { 
      console.log(this, typeof this);
      this._instance = new ExtendedOperators(this._singletonEnforcer);
    }
    return this._instance;
  }
  
  constructor(enforcer) {
    if (enforcer != this.constructor._singletonEnforcer)
      throw new Error("cannot construct singleton. use static method 'instance' instead");
    
    this.clear();
  }
  
  clear() {
    this.operators = {};
    this.allow(SelectCondition.OP_EQ);
  }
  
  // operator should be a SelectCondition.OP_xxx constant
  allow(operator) {
    this.operators[operator] = true;
  }
  
  allowAll() {
    this.operators[SelectCondition.OP_NEQ] = true;
    this.operators[SelectCondition.OP_LT] = true;
    this.operators[SelectCondition.OP_LTE] = true;
    this.operators[SelectCondition.OP_GT] = true;
    this.operators[SelectCondition.OP_GTE] = true;
  }
  
  isAllowed(operator) {
    return this.operators[operator] === true;
  }
}


ExtendedOperators.initConstants();