'use strict'

class NaturalJoin extends Expression {
  constructor( {leftExpression=null, rightExpression=null} = {} ) {
    
    if (ForbiddenOperations.instance.isForbidden("J"))
      throw new AlgebraError(AlgebraError.FORBIDDEN_OPERATION, "Join");
    
    super();
    
    this._left = leftExpression;
    this._right = rightExpression;
    this._leftCommonIndexes = null;
    this._rightCommonIndexes = null;
    this._rightOtherIndexes = null;
    
    this._check();
    this._computeSchema();
    
    super._expressionString = `${leftExpression.toExpressionString(this).trim()} * ${rightExpression.toExpressionString(this).trim()}`;
  }
  
  get level() {
    return 1;
  }
  
  _computeSchema() {
    let leftAttribs = this._left.schema.attribSeq;
    let rightAttribs = this._right.schema.attribSeq;
    
    let attribs = leftAttribs.toArray();
    this._leftCommonIndexes = [];
    this._rightCommonIndexes = [];
    this._rightOtherIndexes = [];
    
    for (let [iR, attrib] of rightAttribs.entries()) {
      let iL = leftAttribs.indexOf(attrib);
      if (iL == -1) {
        attribs.push(attrib);
        this._rightOtherIndexes.push(iR);
      }
      else {
        this._leftCommonIndexes.push(iL);
        this._rightCommonIndexes.push(iR);
      }
    }
    
    this._setSchema(new Schema(attribs));
  }
  
  _evaluate() {
    let tuples = [];
    
    for (let t1 of this._left.tupleSet) {
      for (let t2 of this._right.tupleSet) {
        if (this._matches(t1, t2)) {
          let values = t1.valueSeq.toArray();
          
          for (let i of this._rightOtherIndexes) {
            values.push(t2.getValueAt(i));
          }
          
          tuples.push(new Tuple(values));
        }
      }
    }
    
    this._setTupleSet(new TupleSet(tuples));
    
    delete this._left;
    delete this._right;
  }
  
  _matches(t1, t2) {
    for (let i=0; i<this._leftCommonIndexes.length; i++) {
      let iL = this._leftCommonIndexes[i];
      let iR = this._rightCommonIndexes[i];
      
      if (t1.getValueAt(iL) != t2.getValueAt(iR))
        return false;
    }
    
    return true;
  }
  
  _check() {
    if ( !(this._left instanceof Expression) ) 
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, "left operand");
    
    if ( !(this._right instanceof Expression) ) 
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, "right operand");
  }
}