'use strict'

class Projection extends Expression {
  constructor({
        expression=null, 
        schema=null} = {}) {

    if (ForbiddenOperations.instance.isForbidden("P"))
      throw new AlgebraError(AlgebraError.FORBIDDEN_OPERATION, "Projection");
        
    super();
    this._expression = expression;
    if (schema == null)
      schema = new Schema();      // default : empty schema
    this._setSchema(schema);
    
    this._check();
    
    super._expressionString = `@project{${schema.toString()}}${expression.toExpressionString(this)}`;
  }
  
  get level() {
    return 0;
  }
  
  _evaluate() {
    let indexes = this._expression.schema.getAttributeIndexes(this.schema);
    let projectedTuples = [];
    
    for (let tuple of this._expression.tupleSet) {
      let values = [];
      
      for (let i of indexes) {
        values.push(tuple.getValueAt(i));
      }
      
      projectedTuples.push(new Tuple(values));
    }
    
    
    projectedTuples = TupleSet.removeDuplicates(projectedTuples);
    
    delete this._expression;
    this._setTupleSet(new TupleSet(projectedTuples));
  }
  
  _check() {
    if ( this._expression == null || ! (this._expression instanceof Expression) )
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, this._expression);
    
    for (let attrib of this.schema) {
      if ( ! this._expression.schema.hasAttribute(attrib) )
        throw new AlgebraError(
          AlgebraError.INVALID_SCHEMA,
          `attribute ${attrib} in projection schema is not in expression schema ${this._expression.schema.toString()}` );
    }
  }
}