'use strict'

class Relation extends Expression {
  constructor({
        schema = null, 
        tupleSet = null
      } = {}) {
    super();
    
    if (schema === null)
      schema = new Schema();
    if (tupleSet === null)
      tupleSet = new TupleSet();
    
    if (! (schema instanceof Schema) ){
      throw new AlgebraError(AlgebraError.INVALID_SCHEMA, schema.toString());
    }
    
    this._setSchema(schema);
    this._setTupleSet(tupleSet);
  }
  
  get level() {
    return 0;
  }
  
  get atomic() {
    return true;
  }
  
  // returns a copy of other relation
  clone() {
    let schema = this.schema;
    let tupleSet = this.tupleSet.clone();
    
    return new Relation({schema , tupleSet});
  }
  
  _evaluate(){
    // nothing to evaluate, tuples have been provided at construction
  }
  
  set name(value) {
    super.name = value;
    super._expressionString = value;
  }
  
  // REMARK : when overloading setter, getter must be overloaded too
  get name() {
    return super.name;
  }
}