'use strict'

class Selection extends Expression {
  constructor({expression=null, condition=null} = {}) {
    if (ForbiddenOperations.instance.isForbidden("S"))
      throw new AlgebraError(AlgebraError.FORBIDDEN_OPERATION, "Selection");
    
    super();
    this._expression = expression;
    this._condition = condition;
    
    this._check();
    this._setSchema(expression.schema);
    super._expressionString = `@select{${this._condition.toString()}}${this._expression.toExpressionString(this)}`;
  }
  
  get level() {
    return 0;
  }
  
  _evaluate() {
    let tuples = [];
    
    for (let tuple of this._expression.tupleSet) {
      if ( this._condition.matches(tuple) ) {
        tuples.push(tuple);
      }
    }
    
    delete this._expression;
    this._setTupleSet(new TupleSet(tuples));
  }
  
  _check() {
    if ( this._expression == null || ! (this._expression instanceof Expression) )
      throw new AlgebraError(AlgebraError.INVALID_EXPRESSION, this._expression );
    
    if ( this._condition == null || ! (this._condition instanceof SelectCondition) )
      throw new AlgebraError(AlgebraError.INVALID_CONDITION, this._condition);
    
    this._condition.checkSchemaCompatibility(this._expression.schema);
  }
}