'use strict'

class TupleSet {
  
  static removeDuplicates(arTuples) {
    let seqTuples = Immutable.Seq(arTuples);

    seqTuples = seqTuples.sort(
      ( a,b ) => a.compareTo(b)
    );
    
    let res = [];
    let lastTuple = null;
    for (let tuple of seqTuples.values()) {
      if ( lastTuple == null || ! tuple.equals(lastTuple) ) {
        res.push(tuple);
        lastTuple = tuple;
      }
    }
    
    return res;
  }
  
  static getDuplicates(seqTuples) {
    throw new AlgebraError(AlgebraError.INTERNAL, "not yet implemented");
  }
  
  
  constructor(...args) {
    this._tuples = Immutable.Seq();
    for (let arg of args) {
      this._addArgument(arg);
    }
    this._checkArity();
    this._checkNoDuplicate();
  }
  
  clone() {
    let copy = new TupleSet();
    copy._tuples = this._tuples.map(t => new Tuple(t));
    
    return copy;
  }
  
  get size() {
    return this._tuples.size;
    
  }
  
  contains(tuple) {
    if (! tuple instanceof Tuple)
      throw new AlgebraError(AlgebraError.INVALID_TUPLE, tuple);
    
    return this._tuples.includes(tuple);
  }
  
  equals(other) {
    if (! other instanceof TupleSet)
      throw new AlgebraError(AlgebraError.INVALID_TUPLESET, other);
    
    if (this.size != other.size)
      return false;
    
    if (this.size == 0)
      return true;
    
    return this._tuples.every(
      (t) => other._tuples.contains(t)
    );
  }
  
  getTupleAt(index){
    if (typeof(index) !== "number" || index%1 !== 0)
      throw new AlgebraError(AlgebraError.INVALID_INDEX, `${index} is not an integer`);
    
    if (index <= -1 || index >= this.size)
      throw new AlgebraError(AlgebraError.INVALID_INDEX, `${index} is out of bounds ([${0}..${this.size}[)`);
    
    return this._tuples.get(index);
  }
  
  *[Symbol.iterator](){
    for (let t of this._tuples)
      yield t;
  }
  
  _checkArity(){
    if (this.size <= 1)
      return;
    
    let size = this._tuples.first().size;
    let index = 0;
    for (let t of this._tuples.values()){
      if (t.size !== size)
        throw new AlgebraError(
          AlgebraError.INVALID_TUPLESET, 
          `Different sizes for tuple ${t.toString()} at index ${index} (${t.size}) and previous tuples (${size})`);
      index++;
    }
  }
  
  _checkNoDuplicate(){
    let sorted = this._tuples.sort();
    
    // console.log(sorted.join(", "));
    
    let previous = null;
    for (let current of sorted){
      if (previous != null && current.equals(previous))
        throw new AlgebraError(AlgebraError.INVALID_TUPLESET, `duplicate tuple ${current.toString()}`);
      previous = current;
    }
  }
  
  _addArgument(arg){
    if ( arg === undefined || arg === null )
      throw new AlgebraError(AlgebraError.INVALID_TUPLE, `${arg} is not a tuple`);
    
    if (arg instanceof Array) {
      for (let e of arg) {
        this._addArgument(e);
      }
    }
    else if (arg instanceof Tuple) {
      this._tuples = this._tuples.concat(arg);
    }
    else
      throw new AlgebraError(AlgebraError.INVALID_TUPLE, `${arg} is not a tuple`);
  }
}