class SqfError {
  static _initConstants(){
    SqfError.SYNTAX = Symbol();
    SqfError.SEMANTIC = Symbol();
  }
  
  constructor ( errorType = null, {
      description = null,
      filename=null,
      location=null } = {} ) {
        
    this._errorType = errorType;
    this._description = description;
    this._location = location;
  }
  
  get type() {
    return this._errorType;
  }
  
  get description() {
    return this._description;
  }

  set description(value) {
    this._description = value;
  }
  
  get filename() {
    return this._filename;
  }
  
  set filename(value) {
    this._filename = value;
  }
  
  get location() {
    return this._location;
  }
  
  set location(value) {
    this._location = value;
  }
  
  get column() {
    return this._column;
  }
  
  set column(value) {
    this._column = value;
  }
  
  get description() {
    let res = "";

    switch(this._errorType) {
      case SqfError.SYNTAX:
        res += "Syntax error";
        break;
      case SqfError.SEMANTIC:
        res += "Semantic error";
        break;
      default:
        throw Error("Unexpected SQF error type : " + this._errorType);
        break;
    }
    
    if (this._location != null) {
      let {start, end} = this._location;
      
      res += ` (${start.line}:${start.column} -> ${end.line}:${end.column})`;
      
    }
    
    if (this._description != null) 
      res += `: ${this._description} `;
    
    return res;
  }
  
  toString(){
    return `[SQF] ${this.description}`;
  }
}


SqfError._initConstants();