'use strict'

QUnit.module( "Chaining" );

QUnit.test("exo_wijsen_02", function( assert ) {
/*  
      @define Cities[Name, Country, Population] {
        <Bergen,    Belgium,  20.3>
        <Bergen,    Norway,   30.5>
        <Brussels,  Belgium, 370.6>
      }
*/      
  
  let cities = new Relation({
    schema: new Schema(["Name", "Country", "Population"]),
    tupleSet: new TupleSet([
      new Tuple("Bergen", "Belgium", "20.3"),
      new Tuple("Bergen", "Norway", "30.5"),
      new Tuple("Brussels", "Belgium", "370.6"),
    ]),
  });
  
  cities.name = "Cities";
  console.log(cities.toString());
  
/*
      @define Countries[Name, Capital, Population, Currency] {
        <Belgium,   Brussels,        10 255.6,   EUR>
        <Norway,    Oslo,             4 463.2,   NOK>
        <Japan,     Tokyo,          128 888.0,   YEN>
        <France,    Paris,           10 255.6,   EUR>
        <Salvador,  San Salvador,     6 328.1,   USD> 
        <USA,       Washington,     318 900.3,   USD> 
      }
*/  
  
  let countries = new Relation({
    schema: new Schema(["Name", "Capital", "Population", "Currency"]),
    tupleSet: new TupleSet([
        new Tuple(["Belgium",   "Brussels",        "10 255.6",   "EUR"]),
        new Tuple(["Norway",    "Oslo",             "4 463.2",   "NOK"]),
        new Tuple(["Japan",     "Tokyo",          "128 888.0",   "YEN"]),
        new Tuple(["France",    "Paris",           "10 255.6",   "EUR"]),
        new Tuple(["Salvador",  "San Salvador",     "6 328.1",   "USD"]), 
        new Tuple(["USA",       "Washington",     "318 900.3",   "USD"]), 
    ]),
  });
  
  countries.name = "Countries";
  console.log(countries.toString());
  
/*  
      % Q11. Quelle est la population de la capitale de la Belgique ?
      @let CC = @rename{Name:Country} @project{Name, Capital} Countries
      @let R11 = @project{Population} (
                    @rename{Name:Capital} Cities 
                      @join 
                    @select{Country="Belgium"} CC )
      @print R11
*/

  let cc = new Rename({
    map: {Name: "Country"},
    expression: new Projection({
      schema: new Schema(["Name", "Capital"]),
      expression: countries,
    }),
  });
  
  console.log(cc.toString());
  
  let r11 = new Projection({
    schema: new Schema("Population"),
    expression: new NaturalJoin({
      leftExpression: new Rename({
        map: {Name:"Capital"},
        expression: cities
      }),
      rightExpression: new Selection({
        condition: new SelectCondition({
          left: {attrib: "Country"},
          right: {constant: "Belgium"}
        }),
        expression: cc,
      }),
    }),
  });
  
  r11.name = "R11";
  
  console.log(r11.toString());
  
  assert.equal(r11.tupleSet.size, 1);
  assert.equal(r11.schema.size, 1);
  assert.equal(r11.schema.getAttributeAt(0), "Population");
  assert.equal(r11.tupleSet.getTupleAt(0).getValue("Population"), "370.6");

/*
      % Q12 : Quelle monnaie est utilisée dans plusieurs pays ?
      @let CURRENCIES = @project{Name, Currency} Countries
      @let R12 = @project{Currency} @select{Country<>Name} (
                      @rename{Name:Country} CURRENCIES @join CURRENCIES)
      @print R12


      @print Countries @join Cities, R11  
*/  

  let currencies = new Projection({
    schema: new Schema("Name", "Currency"),
    expression: countries,
  });
  
  console.log(currencies.toString());
  
  let r12 = new Projection({
    schema: new Schema("Currency"),
    expression: new Selection({
      condition: new SelectCondition({
        left: {attrib: "Country"},
        operator: SelectCondition.OP_NEQ,
        right: {attrib: "Name"},
      }),
      expression:   new NaturalJoin({
        leftExpression: new Rename({
          map: {Name: "Country"},
          expression: currencies,
        }),
        rightExpression: currencies,
      }),
    }),
  });
  
  r12.name = "R12",
  console.log(r12.toString());
  
  assert.equal(r12.schema.size, 1);
  assert.equal(r12.schema.getAttributeAt(0), "Currency");
  assert.equal(r12.tupleSet.size, 2);
  assert.ok(r12.tupleSet.contains(new Tuple("EUR")))
  assert.ok(r12.tupleSet.contains(new Tuple("USD")))




  
});

