'use strict'

QUnit.module( "Difference" );

QUnit.test("non empty intersection", function( assert ) {
  let leftSchema = new Schema("id", "prénom", "nom");
  let rightSchema = new Schema("nom", "id", "prénom");
  
  
  let leftTuples = [
    new Tuple("1", "Sergio",      "Mazzoleni"),
    new Tuple("2", "Christophe",  "Dhondt"),
    new Tuple("3", "Emmanuel",    "Duez"),
  ];
  
  let rightTuples = [
    new Tuple("Mazzoleni",      "1", "Sergio"),
    new Tuple("Petit-Barreau",  "4", "Thomas"),
    new Tuple("Deleuze",        "5", "Jonathan"),
  ];
  
  let leftExpr = new Relation({
      schema: leftSchema, 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let rightExpr = new Relation({
      schema: rightSchema, 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  let diff = new Difference({
      leftExpression: leftExpr, 
      rightExpression: rightExpr
  });
  
  assert.equal(diff.tupleSet.size, 2);
  
  console.log(diff.toString());
});



QUnit.test("empty intersection", function( assert ) {
  let leftSchema = new Schema("id", "prénom", "nom");
  let rightSchema = new Schema("nom", "id", "prénom");
  
  
  let leftTuples = [
    new Tuple("1", "Sergio",      "Mazzoleni"),
    new Tuple("2", "Christophe",  "Dhondt"),
    new Tuple("3", "Emmanuel",    "Duez"),
  ];
  
  let rightTuples = [
    new Tuple("Ogier",          "6", "Marc"),
    new Tuple("Petit-Barreau",  "4", "Thomas"),
    new Tuple("Deleuze",        "5", "Jonathan"),
  ];
  
  let leftExpr = new Relation({
      schema: leftSchema, 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let rightExpr = new Relation({
      schema: rightSchema, 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  let diff = new Difference({
    leftExpression: leftExpr, 
    rightExpression: rightExpr
  });
  
  assert.equal(diff.tupleSet.size, 3);
  assert.ok(diff.tupleSet.equals(leftExpr.tupleSet));
  
  console.log(diff.toString());
});


QUnit.test("with empty relations", function( assert ) {
  let leftSchema = new Schema("id", "prénom", "nom");
  let rightSchema = new Schema("nom", "id", "prénom");
  
  
  let leftTuples = [
    new Tuple("1", "Sergio",      "Mazzoleni"),
    new Tuple("2", "Christophe",  "Dhondt"),
    new Tuple("3", "Emmanuel",    "Duez"),
  ];
  
  let rightTuples = [
  ];
  
  let leftExpr = new Relation({
      schema: leftSchema, 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let rightExpr = new Relation({
      schema: rightSchema, 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  let diff = new Difference({
    leftExpression: leftExpr, 
    rightExpression: rightExpr
  });
  
  assert.equal(diff.tupleSet.size, 3);
  assert.ok(diff.tupleSet.equals(leftExpr.tupleSet));
  
  console.log(diff.toString());
  
  
  diff = new Difference({
    leftExpression: rightExpr, 
    rightExpression: leftExpr
  });
  assert.equal(diff.tupleSet.size, 0);
  assert.ok(diff.tupleSet.equals(rightExpr.tupleSet));
  
  console.log(diff.toString());
});




QUnit.test("with itself", function( assert ) {
  let schema = new Schema("id", "prénom", "nom");
  
  
  let leftTuples = [
    new Tuple("1", "Sergio",      "Mazzoleni"),
    new Tuple("2", "Christophe",  "Dhondt"),
    new Tuple("3", "Emmanuel",    "Duez"),
  ];
  
  let rightTuples = [
    new Tuple("1", "Sergio",      "Mazzoleni"),
    new Tuple("2", "Christophe",  "Dhondt"),
    new Tuple("3", "Emmanuel",    "Duez"),
  ];
  
  
  let leftExpr = new Relation({
      schema, 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let rightExpr = new Relation({
      schema, 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  // different instances of the same expression
  let diff = new Difference({
    leftExpression: leftExpr, 
    rightExpression: rightExpr
  });
  assert.equal(diff.tupleSet.size, 0);
  console.log(diff.toString());

  // same instance
  diff = new Difference({
    leftExpression: leftExpr, 
    rightExpression: leftExpr
  });
  assert.equal(diff.tupleSet.size, 0);
  console.log(diff.toString());
});




QUnit.test("invalid expression", function( assert ) {
  assert.throws(
    () => { 
      new Difference() 
    },
    function (err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_EXPRESSION;
    },
    "missing expression operands"
  );
  
  let r = new Relation();
  
  assert.throws(
    () => { 
      new Difference({leftExpression:r}) 
    },
    function (err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_EXPRESSION;
    },
    "missing expression operands"
  );
  
  assert.throws(
    () => { 
      new Difference({rightExpression:r}) 
    },
    function (err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_EXPRESSION;
    },
    "missing expression operands"
  );
  
  
  
});




