'use strict'

QUnit.module( "Extended opertators" );

QUnit.test("singleton instance", function( assert ) {
  
  let inst = ExtendedOperators.instance;
  
  inst.clear();
  
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_EQ));
  assert.ok(!inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
});

QUnit.test("instantiation", function( assert ) {
  assert.throws(
    () => {
      let o = new ExtendedOperators();
    },
    Error,
    "cannot instanciate singleton"
  );
});


QUnit.test("allowing operators", function( assert ) {
  
  let inst = ExtendedOperators.instance;
  
  inst.clear();
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(!inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
  
  inst.allow(SelectCondition.OP_NEQ);
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
  
  inst.allow(SelectCondition.OP_LT);
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
  
  inst.allow(SelectCondition.OP_LTE);
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
  
  inst.allow(SelectCondition.OP_GT);
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(!inst.isAllowed(SelectCondition.OP_GTE));
  
  inst.allow(SelectCondition.OP_GTE);
  assert.ok(inst instanceof ExtendedOperators);
  assert.ok(inst.isAllowed(SelectCondition.OP_NEQ));
  assert.ok(inst.isAllowed(SelectCondition.OP_LT));
  assert.ok(inst.isAllowed(SelectCondition.OP_LTE));
  assert.ok(inst.isAllowed(SelectCondition.OP_GT));
  assert.ok(inst.isAllowed(SelectCondition.OP_GTE));
});
