'use strict'

QUnit.module( "NaturalJoin" );

QUnit.test("Natural Join with common attributes", function( assert ) {
  let leftAttribs = ["id", "prénom", "nom"];
  let rightAttribs = ["id", "cours", "points"];
  
  let leftTuples = [
    new Tuple("1", "Sergio", "Mazzoleni"),
    new Tuple("2", "Christophe", "Dhondt"),
    new Tuple("3", "Emmanuel", "Duez"),
  ];
  
  let rightTuples = [
    new Tuple("1", "DB II", "19.5"),
    new Tuple("2", "DB II", "17.0"),
    new Tuple("3", "DB II", "9.0"),
    new Tuple("1", "Rech op", "20"),
    new Tuple("2", "Rech op", "11.0"),
  ];
  
  let leftExpression = new Relation({
      schema:new Schema(leftAttribs), 
      tupleSet: new TupleSet(leftTuples),
  });
  
  let rightExpression = new Relation({
      schema: new Schema(rightAttribs), 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  let join = new NaturalJoin({leftExpression, rightExpression});
  
  assert.equal(join.tupleSet.size, 5);
  
  for (let t of join.tupleSet) {
    console.log(t.toString());
  }
});


QUnit.test("Natural Join with only distinct attributes", function( assert ) {
  let leftAttribs = ["prénom", "nom"];
  let rightAttribs = ["cours", "prof"];
  
  let leftTuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Emmanuel", "Duez"),
  ];
  
  let rightTuples = [
    new Tuple("DB II", "Wijsen"),
    new Tuple("Structure de données", "Bruyère"),
    new Tuple("Recherche opérationnelle", "Fortz"),
    new Tuple("IA", "Mélot"),
  ];
  
  let leftExpression = new Relation({
      schema: new Schema(leftAttribs), 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let rightExpression = new Relation({
      schema: new Schema(rightAttribs), 
      tupleSet: new TupleSet(rightTuples) 
  });
      
  let join = new NaturalJoin({leftExpression, rightExpression});
  
  assert.equal(join.tupleSet.size, leftTuples.length*rightTuples.length);
  
  for (let t of join.tupleSet) {
    console.log(t.toString());
  }
});


QUnit.test("Natural Join with itself", function( assert ) {
  let leftAttribs = ["prénom", "nom"];
  let rightAttribs = ["cours", "prof"];
  
  let leftTuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Emmanuel", "Duez"),
  ];
  
  let leftExpr = new Relation({
      schema: new Schema(leftAttribs), 
      tupleSet: new TupleSet(leftTuples) 
  });
  
  let join = new NaturalJoin({
    leftExpression: leftExpr, 
    rightExpression: leftExpr,
  });
  
  assert.ok(leftExpr.tupleSet.equals(join.tupleSet));
  
  
  for (let t of join.tupleSet) {
    console.log(t.toString());
  }
});




