'use strict'

QUnit.module( "Projection" );

QUnit.test("Projection on first attribute", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet:new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let pSchema = new Schema("prénom");
  
  let p = new Projection({expression: r, schema:pSchema});
  assert.equal(p.tupleSet.size, 4);
  
  assert.ok(p.tupleSet.contains(new Tuple("Sergio")));
  assert.ok(p.tupleSet.contains(new Tuple("Isabelle")));
  assert.ok(p.tupleSet.contains(new Tuple("Christophe")));
  assert.ok(p.tupleSet.contains(new Tuple("Eugenio")));
  
  for (let t of p.tupleSet) {
    console.log(t.toString());
    assert.ok(t.schema != null);
    assert.ok(t.schema.equals(pSchema));
  }
});


QUnit.test("Projection on second attribute", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet:new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let p = new Projection({expression:r, schema:new Schema("nom")});
  assert.equal(p.tupleSet.size, 5);
  
  assert.ok(p.tupleSet.contains(new Tuple("Mazzoleni")));
  assert.ok(p.tupleSet.contains(new Tuple("Dello Strito")));
  assert.ok(p.tupleSet.contains(new Tuple("Raes")));
  assert.ok(p.tupleSet.contains(new Tuple("Dhondt")));
  assert.ok(p.tupleSet.contains(new Tuple("Leone")));
  
  for (let t of p.tupleSet) {
    console.log(t.toString());
    assert.ok(t.schema != null);
    assert.ok(t.schema.equals(new Schema("nom")));
  }
});



QUnit.test("Projection with empty schema", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet: new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let p = new Projection({expression:r, schema:new Schema()});
  assert.equal(p.tupleSet.size, 1);
  
  assert.ok(p.tupleSet.contains(new Tuple()));
  
  for (let t of p.tupleSet) {
    console.log(t.toString());
    assert.ok(t.schema != null);
    assert.ok(t.schema.equals(new Schema()));
  }
});


QUnit.test("Projection with empty schema on empty relation", function( assert ) {
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet:new TupleSet()});
  
  assert.equal(r.tupleSet.size, 0);
  
  let p = new Projection({expression:r, schema:new Schema()});
  assert.equal(p.tupleSet.size, 0);
  
  assert.notOk(p.tupleSet.contains(new Tuple()));
  
  for (let t of p.tupleSet) {
    console.log(t.toString());
    assert.ok(t.schema != null);
    assert.ok(t.schema.equals(new Schema()));
  }
});


QUnit.test("Projection using same (instance of) schema", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet:new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let p = new Projection({expression:r, schema});
  assert.equal(p.tupleSet.size, 6);
  
  assert.ok(r.tupleSet.equals(p.tupleSet));
});




