'use strict'

QUnit.module( "Relation" );

QUnit.test("constructor default", function( assert ) {
  let r;

  r = new Relation();
  assert.equal(r.schema.size, 0);
  assert.equal(r.tupleSet.size, 0);
  
  r = new Relation(new Schema());
  assert.equal(r.schema.size, 0);
  assert.equal(r.tupleSet.size, 0);
  
  r = new Relation(new Schema(), new TupleSet());
  assert.equal(r.schema.size, 0);
  assert.equal(r.tupleSet.size, 0);
});


QUnit.test("constructor schema only", function( assert ) {
  let r = new Relation({
    schema: new Schema("a1", "a2"),
  });
  
  assert.equal(r.schema.size, 2);
  assert.equal(r.tupleSet.size, 0);
});

QUnit.test("constructor schema and tupleset", function( assert ) {
  let r = new Relation({
    schema: new Schema("Prénom", "Nom"),
    tupleSet: new TupleSet(
      new Tuple("Isabelle", "Dello Strito"),
      new Tuple("Sergio", "Mazzoleni"),
      new Tuple("Christophe", "Raes"),
      new Tuple("Caroline", "Danloy")
    )
  });
  
  assert.equal(r.schema.size, 2);
  assert.equal(r.tupleSet.size, 4);
  
  let empty = new Relation({
    schema: null, 
    tupleSet: new TupleSet(new Tuple())
  });
  assert.equal(empty.schema.size, 0);
  assert.equal(empty.tupleSet.size, 1, "empty tuple");
});

QUnit.test("relation no schema + empty tupleset", function( assert ) {
  let r = new Relation({tupleSet: new TupleSet()});
  assert.equal(r.tupleSet.size, 0);
  assert.equal(r.schema.size, 0);
});

QUnit.test("relation no schema + tupleset with unique tuple", function( assert ) {
  let r = new Relation({tupleSet: new TupleSet(new Tuple())});
  assert.equal(r.tupleSet.size, 1);
  assert.equal(r.schema.size, 0);
});


QUnit.test("constructor errors", function( assert ) {
  assert.throws(
    () => {
      new Relation({
        schema: new Schema("Prénom", "Nom", "Adresse"),
        tupleSet: new TupleSet(
          new Tuple("Isabelle", "Dello Strito")
        )
      });
    },
    function(err){
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_SCHEMA;
    },
    "Schema and tuples with different arities"
  );
});


QUnit.test("cloning relation", function( assert ) {
  
  let r = new Relation({
    schema: new Schema("Prénom", "Nom"),
    tupleSet: new TupleSet(
      new Tuple("Isabelle", "Dello Strito"),
      new Tuple("Sergio", "Mazzoleni"),
      new Tuple("Christophe", "Raes"),
      new Tuple("Caroline", "Danloy")
    )
  });
  
  let copy = r.clone();
  
  assert.equal(r.schema.size, 2);
  assert.equal(r.tupleSet.size, 4);

  assert.equal(copy.schema.size, r.schema.size);
  assert.equal(copy.tupleSet.size, r.tupleSet.size);

});
