'use strict'


QUnit.module( "Rename" );


QUnit.test("Full example", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet:new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let rename = new Rename({
    expression:r, 
    map: {prénom: "firstname", nom:"lastname"}
  });
  
  assert.ok(rename.schema.equals(new Schema("firstname", "lastname")));
  assert.ok(rename.schema.equals(new Schema("lastname", "firstname")));
  
  assert.equal(r.tupleSet.size, rename.tupleSet.size);
  
  console.log(rename.toString());
  console.log(rename);
});


QUnit.test("Rename with duplicated targets", function( assert ) {
  let schema = new Schema("a1", "a2", "a3");
  
  let r = new Relation({schema});
  
  assert.throws(
    () => {
      new Rename({
        expression: r, 
        map: {a1:"target", a3:"anything", a2:"target"}
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_RENAME_MAP;
    },
    "same target name used several times"
  );
});

QUnit.test("Rename with unknown attribute", function( assert ) {
  let schema = new Schema("a1", "a2", "a3");
  
  let r = new Relation({schema});
  
  assert.throws(
    () => {
      new Rename({
        expression: r, 
        map: {a1:"something", b2:"anything", a3:"something else"}
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_RENAME_MAP;
    },
    "'b2' is not part of the schema"
  );
});


QUnit.test("Rename with target names already present in schema", function( assert ) {
  let schema = new Schema("a1", "a2", "a3");
  
  let r = new Relation({schema});
  
  assert.equal(r.tupleSet.size, 0);
  
  assert.throws(
    () => {
      new Rename({
        expression:r, 
        map: {a1:"a3"}
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_RENAME_MAP;
    },
    "'a3' is already part of schema and is not renamed to something else"
  );
});


QUnit.test("Rename with swapping attributes", function( assert ) {
  let schema = new Schema("a1", "a2", "a3");
  
  let r = new Relation({schema});
  
  let rename = new Rename({
    expression: r, 
    map: {a1:"a2", a2:"a1"}
  });
  
  assert.equal(rename.paramCount, 2, "swapping element is ok");
  assert.ok(rename.schema.equals(schema));
  
});



QUnit.test("Rename with 'noise' in param object", function( assert ) {
  let schema = new Schema("a1", "a2", "a3");
  
  let r = new Relation({schema});
  
  // property whose value is not a string is discarded
  let rename = new Rename({
    expression: r, 
    map: {
      a1: "newName",
      noise1: null,
      noise2: 10,
      noise3: undefined,
      someMethod: function() {return "newName";},
    },
  });
  
  assert.equal(rename.paramCount, 1);
  console.log(rename.schema.toString());
  assert.ok(rename.schema.equals(new Schema("newName", "a2", "a3")));
});



