'use strict'

QUnit.module( "SelectCondition" );


QUnit.test("constructor attrib [EQ] attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
  };
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("same", "same", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("same", "different", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));
});



QUnit.test("constructor attrib EQ attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_EQ,
  };
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("same", "same", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("same", "different", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));
  
});


QUnit.test("constructor attrib NEQ attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_NEQ,
  };
  
  ExtendedOperators.instance.allow(SelectCondition.OP_NEQ);
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("same", "same", "whatever");
  t1.schema = s;
  assert.notOk(condition.matches(t1));
  
  let t2 = new Tuple("same", "different", "whatever");
  t2.schema = s;
  assert.ok(condition.matches(t2));
});


QUnit.test("constructor attrib LT attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_LT,
  };
  
  ExtendedOperators.instance.allow(SelectCondition.OP_LT);
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("B", "A", "whatever");
  t1.schema = s;
  assert.notOk(condition.matches(t1));
  
  let t2 = new Tuple("B", "B", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));

  let t3 = new Tuple("B", "C", "whatever");
  t3.schema = s;
  assert.ok(condition.matches(t3));
});


QUnit.test("constructor attrib LTE attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_LTE,
  };
  
  ExtendedOperators.instance.allow(SelectCondition.OP_LTE);
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("B", "A", "whatever");
  t1.schema = s;
  assert.notOk(condition.matches(t1));
  
  let t2 = new Tuple("B", "B", "whatever");
  t2.schema = s;
  assert.ok(condition.matches(t2));

  let t3 = new Tuple("B", "C", "whatever");
  t3.schema = s;
  assert.ok(condition.matches(t3));
});


QUnit.test("constructor attrib GT attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_GT,
  };
  
  ExtendedOperators.instance.allow(SelectCondition.OP_GT);
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("B", "A", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("B", "B", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));

  let t3 = new Tuple("B", "C", "whatever");
  t3.schema = s;
  assert.notOk(condition.matches(t3));
});


QUnit.test("constructor attrib GTE attrib", function( assert ) {

  let options = {
    left:{attrib: "a1"},
    right:{attrib: "a2"},
    operator: SelectCondition.OP_GTE,
  };
  
  ExtendedOperators.instance.allow(SelectCondition.OP_GTE);
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("B", "A", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("B", "B", "whatever");
  t2.schema = s;
  assert.ok(condition.matches(t2));

  let t3 = new Tuple("B", "C", "whatever");
  t3.schema = s;
  assert.notOk(condition.matches(t3));
});




QUnit.test("constructor constant [EQ] attrib", function( assert ) {
  
  let options = {
    left:{constant: "val1"},
    right:{attrib: "a2"},
  };
  
  let condition = new SelectCondition(options);
  
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("whatever", "val1", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("whatever", "different", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));
  
  assert.ok(true);
});


QUnit.test("constructor attrib [EQ] constant", function( assert ) {
  
  let options = {
    left:{attrib: "a2"},
    right:{constant: "some value"},
  };
  
  let condition = new SelectCondition(options);
  
  let s = new Schema("a1", "a2", "a3");
  
  let t1 = new Tuple("whatever", "some value", "whatever");
  t1.schema = s;
  assert.ok(condition.matches(t1));
  
  let t2 = new Tuple("whatever", "different value", "whatever");
  t2.schema = s;
  assert.notOk(condition.matches(t2));
  
  assert.ok(true);
});



QUnit.test("invalid constructor call", function( assert ) {
  assert.throws(
    ()=>{
      let c = new SelectCondition();
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "no argument provided"
  );

  assert.throws(
    ()=>{
      let c = new SelectCondition({
        right:{attrib: "a1"}
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "no left operand provided"
  );

  assert.throws(
    ()=>{
      let c = new SelectCondition({
        left:{attrib: "a1"}
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "no right operand provided"
  );

  assert.throws(
    ()=>{
      let c = new SelectCondition({
        left:"a1",
        right:"a2",
      });
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "argument must have a property 'attrib' or 'constant'"
  );

  assert.throws(
    ()=>{
      let options = {
        left:{constant: "val1"},
        right:{constant: "val2"},
      };
      
      let condition = new SelectCondition(options);
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "at least one operand must be an attribute"
  );

  assert.throws(
    ()=>{
      let options = {
        left:{attrib: "a1"},
        right:{attrib: "a2"},
        operator: "equals",
      };
      
      let condition = new SelectCondition(options);
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "operator must be one of the SelectCondition.OP_xxx constants"
  );
});
