'use strict'

QUnit.module( "Selection" );

QUnit.test("Selection constructor", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({
    schema, 
    tupleSet: new TupleSet(tuples),
  });
  
  assert.equal(r.tupleSet.size, 6);
  
  let cond = new SelectCondition({
      left: {attrib:"prénom"},
      right: {constant:"Sergio"},
  });
  
  let select = new Selection({expression:r, condition:cond});
  
  assert.equal(select.tupleSet.size, 2);
  assert.ok(select.tupleSet.contains(new Tuple("Sergio", "Mazzoleni")));
  assert.notOk(select.tupleSet.contains(new Tuple("Eugenio", "Mazzoleni")));
  
  
  for (let t of select.tupleSet) {
    console.log(t.toString());
  }
});


QUnit.test("Empty selection", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({
    schema, 
    tupleSet: new TupleSet(tuples)
  });
  
  assert.equal(r.tupleSet.size, 6);
  
  let cond = new SelectCondition({
      left: {attrib:"prénom"},
      right: {constant:"Martin"},
  });
  
  let select = new Selection({expression:r, condition:cond});
  
  assert.equal(select.tupleSet.size, 0);

  for (let t of select.tupleSet) {
    console.log(t.toString());
  }
});



QUnit.test("Selection with invalid condition", function( assert ) {
  let tuples = [
    new Tuple("Sergio", "Mazzoleni"),
    new Tuple("Isabelle", "Dello Strito"),
    new Tuple("Christophe", "Raes"),
    new Tuple("Christophe", "Dhondt"),
    new Tuple("Sergio", "Leone"),
    new Tuple("Eugenio", "Mazzoleni"),
  ];
  
  let schema = new Schema("prénom", "nom");
  
  let r = new Relation({schema, tupleSet: new TupleSet(tuples)});
  
  assert.equal(r.tupleSet.size, 6);
  
  let cond = new SelectCondition({
      left: {attrib:"ville"},
      right: {constant:"La Louvière"},
  });
  
  assert.throws(
    () => { 
      new Selection({expression:r, condition:cond});
    },
    function(err) {
      console.log(err.toString());
      return err.type == AlgebraError.INVALID_CONDITION;
    },
    "no attribute 'ville' in schema"
  );
});