'use strict'

QUnit.module( "TupleSet" );

QUnit.test("constructor", function( assert ) {
  let set = new TupleSet();
  assert.equal(set.size, 0);
});

QUnit.test("constructor", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello"));
  tuples.push(new Tuple("Christophe", "Raes"));
  
  set = new TupleSet(tuples);
  
  assert.equal(set.size, 3);
});

QUnit.test("constructor two tuples with different values", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello", "Strito"));
  
  assert.throws(
    ()=>{
      set = new TupleSet(tuples);
    },
    function(err){
      assert.ok(err instanceof AlgebraError);
      console.log(err.toString());
      assert.equal(err.type, AlgebraError.INVALID_TUPLESET);
      return true;
    },
    "different arites"
  );
});

QUnit.test("constructor with duplicate tuple", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello"));
  tuples.push(new Tuple("Christophe", "Raes"));
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  
  assert.throws(
    ()=>{
      set = new TupleSet(tuples);
    },
    function(err){
      assert.ok(err instanceof AlgebraError);
      console.log(err.toString());
      assert.equal(err.type, AlgebraError.INVALID_TUPLESET);
      return true;
    },
    "duplicate tuple"
  );
});


QUnit.test("contains", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello"));
  tuples.push(new Tuple("Christophe", "Raes"));
  
  set = new TupleSet(tuples);
  
  assert.ok(set.contains(new Tuple("Sergio", "Mazzoleni")));
  assert.ok(set.contains(new Tuple("Isabelle", "Dello")));
  assert.ok(set.contains(new Tuple("Christophe", "Raes")));
  assert.notOk(set.contains(new Tuple("Pascal", "Durlet")));
});

QUnit.test("equals", function( assert ) {
  let set1, set2;
  let tuples1 = [];
  let tuples2 = [];
  
  tuples1.push(new Tuple("Sergio", "Mazzoleni"));
  tuples1.push(new Tuple("Isabelle", "Dello"));
  tuples1.push(new Tuple("Christophe", "Raes"));


  tuples2.push(new Tuple("Isabelle", "Dello"));
  tuples2.push(new Tuple("Christophe", "Raes"));
  tuples2.push(new Tuple("Sergio", "Mazzoleni"));
  
  set1 = new TupleSet(tuples1);
  set2 = new TupleSet(tuples2);
  
  assert.ok(set1.equals(set2));
  assert.ok(set2.equals(set1));
});


QUnit.test("equals", function( assert ) {
  let set1, set2;
  let tuples1 = [];
  let tuples2 = [];
  
  tuples1.push(new Tuple("Sergio", "Mazzoleni"));
  tuples1.push(new Tuple("Isabelle", "Dello"));

  tuples2.push(new Tuple("Isabelle", "Dello"));
  tuples2.push(new Tuple("Christophe", "Raes"));
  tuples2.push(new Tuple("Sergio", "Mazzoleni"));
  
  set1 = new TupleSet(tuples1);
  set2 = new TupleSet(tuples2);
  
  assert.notOk(set1.equals(set2));
  assert.notOk(set2.equals(set1));
});

QUnit.test("equals", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello"));
  tuples.push(new Tuple("Christophe", "Raes"));

  set = new TupleSet(tuples);
  
  assert.ok(set.equals(set), "set equals to itself");
});


QUnit.test("equals", function( assert ) {
  let set1 = new TupleSet();
  let set2 = new TupleSet();
  
  assert.ok(set1.equals(set2), "empty sets are equal");
});


QUnit.test("iterator", function( assert ) {
  let set;
  let tuples = [];
  
  tuples.push(new Tuple("Sergio", "Mazzoleni"));
  tuples.push(new Tuple("Isabelle", "Dello"));
  tuples.push(new Tuple("Christophe", "Raes"));
  
  set = new TupleSet(tuples);
  
  let i = 0;
  for (let t of set) {
    assert.ok(t.equals(tuples[i]));
    i++;
  }
  assert.equal(i, tuples.length);
});



QUnit.test("contains", function( assert ) {
  let set = new TupleSet([
    new Tuple("T1"),
    new Tuple("T2"),
    new Tuple("T3"),
    new Tuple("T4"),
  ]);
  
  assert.ok(set.getTupleAt(0) !== null);
  assert.ok(set.getTupleAt(1) !== null);
  assert.ok(set.getTupleAt(2) !== null);
  assert.ok(set.getTupleAt(3) !== null);

  assert.ok(set.getTupleAt(0).equals(new Tuple("T1")));
  assert.ok(set.getTupleAt(1).equals(new Tuple("T2")));
  assert.ok(set.getTupleAt(2).equals(new Tuple("T3")));
  assert.ok(set.getTupleAt(3).equals(new Tuple("T4")));
  
  assert.throws(()=>{set.getTupleAt(-2)}, "out of bounds index");
  assert.throws(()=>{set.getTupleAt(-1)}, "out of bounds index");
  assert.throws(()=>{set.getTupleAt(4)}, "out of bounds index");
  assert.throws(()=>{set.getTupleAt(5)}, "out of bounds index");
  
  assert.throws(()=>{set.getTupleAt()}, "missing argument");
  assert.throws(()=>{set.getTupleAt("hello")}, "wrong argument type");
  
  
});


