'use strict'


Parser.load('./parser.peg', parser => {
  let test = ["a", "b", "c", "d"];
  for (let x of test)
    console.log("test", x);
  
  QUnit.module( "Extended operators" );
  
  QUnit.test("allow none", function( assert ) {
    let source = `@allow none`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow all", function( assert ) {
    let source = `@allow all`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow NEQ", function( assert ) {
    let source = `@allow <>`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow LT", function( assert ) {
    let source = `@allow <`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow LTE", function( assert ) {
    let source = `@allow <=`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow GT", function( assert ) {
    let source = `@allow >`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow GTE", function( assert ) {
    let source = `@allow >=`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("allow all operators", function( assert ) {
    let source = `@allow < > <> >= <=`;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(ops.isAllowed(SelectCondition.OP_GTE));
  });
  
  QUnit.test("last allow wins", function( assert ) {
    let source = `
      @allow < > <> >= <=
      @allow <
    `;
    let {database:db} = parser.parse(source);
    
    let ops = ExtendedOperators.instance;
    assert.ok(ops.isAllowed(SelectCondition.OP_EQ));
    assert.ok(!ops.isAllowed(SelectCondition.OP_NEQ));
    assert.ok(ops.isAllowed(SelectCondition.OP_LT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_LTE));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GT));
    assert.ok(!ops.isAllowed(SelectCondition.OP_GTE));
  });
  
});





