'use strict'


Parser.load('./parser.peg', parser => {
  
  QUnit.module( "precedence" );
  
  
  QUnit.test("union, difference, join", function( assert ) {
    let source = `
      @let A1 = @relation[a1, a2, a3] {}
      @let A2 = @relation[a1, a2, a3] {}
      @let A3 = @relation[a1, a2, a3] {}

      @let r1 = A1 * A2 * A3
      @let r2_1 = A1 + A2 + A3
      @let r2_2 = A1 - A2 - A3
      @let r3 = A1 - A2 + A3
      @let r4 = A1 + A2 - A3
      @let r5_1 = A1 * A2 + A3
      @let r5_2 = A1 + A2 * A3

      @let r6_1 = A1 * (A2 + A3)
      @let r6_2 = (A1 + A2) * A3
      @let r6_3 = A1 * (A2 - A3)
      @let r6_4 = (A1 - A2) * A3
`
    
    let {database:db} = parser.parse(source);
    
    // console.log(db.toString());
    
    let expected = {
      r1  : "(A1*A2)*A3",
      r2_1: "(A1+A2)+A3",
      r2_2: "(A1-A2)-A3",
      r3  : "(A1-A2)+A3",
      r4  : "(A1+A2)-A3",
      r5_1: "(A1*A2)+A3",
      r5_2: "A1+(A2*A3)",
      r6_1: "A1*(A2+A3)",
      r6_2: "(A1+A2)*A3",
      r6_3: "A1*(A2-A3)",
      r6_4: "(A1-A2)*A3",
    };
    
    for (let name of Object.getOwnPropertyNames(expected)) {
      assert.equal(db.get(name).toExpressionString().replace(/ /g, ""), expected[name]);
    }
  });



  QUnit.test("select join", function( assert ) {
    let source = `
      @let A1 = @relation[a1, a2, a3] {}
      @let A2 = @relation[a1, a2, a3] {}
      @let A3 = @relation[a1, a2, a3] {}

      @let e = @select{a1 = a2} A1 * A2 * A3
`
    
    let {database:db} = parser.parse(source);
    assert.equal(db.size, 4);
    
    console.log(db.get("e").toExpressionString());
    assert.equal(db.get("e").toExpressionString(), "((@select{a1=a2} A1) * A2) * A3");
  });
  
  QUnit.test("select union select", function( assert ) {
    let source = `
      @let A1 = @relation[a1, a2, a3] {}
      @let A2 = @relation[a1, a2, a3] {}
      @let A3 = @relation[a1, a2, a3] {}

      @let e = @select{a1 = a2} A1 + @select{a1=a2} A2
`
    
    let {database:db} = parser.parse(source);
    assert.equal(db.size, 4);
    
    console.log(db.get("e").toExpressionString());
    assert.equal(db.get("e").toExpressionString(), "(@select{a1=a2} A1) + (@select{a1=a2} A2)");
  });
});






