'use strict'


Parser.load('./parser.peg', parser => {
  
  QUnit.module( "Relation" );
  
  QUnit.test("no attribute, no tuple", function( assert ) {
    let source = `@let r = @relation[] { }`
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 0);
    assert.equal(r.tupleSet.size, 0);
  });
  
  
  QUnit.test("no attribute, one tuple", function( assert ) {
    let source = `@let r = @relation[]{<>}`;
    console.log("source: ´" + source + "´");
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 0);
    assert.equal(r.tupleSet.size, 1);
  });
  
  QUnit.test("one attribute, one tuple with no value", function( assert ) {
    let source = `@let r = @relation[a1]{<>}`;
    console.log("source: ´" + source + "´");
    
    assert.throws(
      () => { parser.parse(source) },
      function(err) {
        console.log(err.toString());
        assert.equal(err.type, SqfError.SEMANTIC);
        assert.ok(err.description.includes("invalid schema"));
        return true;
      },
      "schema with one attribute and tuple with no value"
    );
  });
  
  QUnit.test("no attribute, one tuple with extra spaces", function( assert ) {
    let source = `@let r = @relation[]{<   >}`;
    console.log("source: ´" + source + "´");
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 0);
    assert.equal(r.tupleSet.size, 1);
  });
  

  QUnit.test("one attribute, three tuples", function( assert ) {
    let source = `@let r = @relation[a1]{
      <t1>
      <t2>
      <t3>
    }`;
    
    console.log("source: ´" + source + "´");
    
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 1);
    assert.equal(r.tupleSet.size, 3);
  });


  QUnit.test("duplicate tuple", function( assert ) {
    let source = `@let r = @relation[a1]{
      <t1>
      <t2>
      <t1>
    }`;
    
    console.log(source);
    
    assert.throws(
      () => { parser.parse(source) },
      function(err) {
        console.log(err.toString());
        assert.equal(err.type, SqfError.SEMANTIC);
        assert.ok(err.description.includes("duplicate tuple"));
        return true;
      },
      "duplicate tuple"
    );
  });
  
  QUnit.test("different schema and tuple size", function( assert ) {
    let source = `@let r = @relation[a1]{
      <t1>
      <t2, extra value>
      <t3>
    }`;
    
    assert.throws(
      () => { parser.parse(source) },
      function(err) {
        console.log(err.toString());
        assert.equal(err.type, SqfError.SEMANTIC);
        assert.ok(err.description.includes("invalid tuple set"));
        return true;
      },
      "second tuple has two values while schema has only one attribute"
    );
  });
  
  QUnit.test("using same expression name several times", function( assert ) {
    let source = 
      ` @let r = @relation[a1] { }
        @let r = @relation[a2] { } `
        
    assert.throws(
      () => { parser.parse(source) },
      function(err) {
        console.log(err.toString());
        assert.equal(err.type, SqfError.SEMANTIC, "semantic error");
        assert.ok(err.description.search(/expression .* already defined/) !== -1, "error message");
        return true;
      },
      "error on using same expression name several times"
    );
  });
  
  
});





