'use strict'


Parser.load('./parser.peg', parser => {
  
  QUnit.module( "Schema" );
  
  QUnit.test("three attributes - standard spacing", function( assert ) {
    let source = `@let r = @relation[a1, a2, a3] { }`
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 3);
  });
  
  QUnit.test("three attributes - no spacing", function( assert ) {
    let source = `@let r = @relation[a1,a2,a3] { }`
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 3);
  });
  
  QUnit.test("three attributes - extra spacing", function( assert ) {
    let source = `@let r = @relation[   a1   ,       a2  ,   a3     ] { }`
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 3);
  });
  
  QUnit.test("three attributes - extra spacing (multiline)", function( assert ) {
    let source = `@let r = @relation[
          a1,
          a2,
          a3     
    ] { 
    }`
    let {database:db} = parser.parse(source);
    
    console.log(db.toString());
    
    assert.equal(db.size, 1);
    let r = db.get("r");
    
    assert.ok(r != null);
    assert.equal(r.schema.size, 3);
  });
  

  
  QUnit.test("duplicate attribute", function( assert ) {
    let source = `@let r = @relation[a1, a2, a1] { }`;

    assert.throws(
      () => { parser.parse(source) },
      function(err) {
        console.log(err.toString());
        assert.equal(err.type, SqfError.SEMANTIC);
        assert.ok(err.description.includes(""));
        return true;
      },
      "duplicate attribute 'a1'"
    );
  });
  
  
  
});





